@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: GlobalHistory.cpp,v 1.23 2004/07/30 19:56:08 chpe Exp $
+ *  $Id: GlobalHistory.cpp,v 1.26.4.3 2004/09/25 13:23:48 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -43,8 +43,13 @@ MozGlobalHistory::~MozGlobalHistory ()
 {
 }
 
+#ifdef MOZ_NSIBROWSERHISTORY_ADDURI_WITH_REFERRER
+/* void addURI (in nsIURI aURI, in boolean aRedirect, in boolean aToplevel, in nsIURI aReferrer); */
+NS_IMETHODIMP MozGlobalHistory::AddURI(nsIURI *aURI, PRBool aRedirect, PRBool aToplevel, nsIURI *aReferrer)
+#else
 /* void addURI (in nsIURI aURI, in boolean aRedirect, in boolean aToplevel); */
 NS_IMETHODIMP MozGlobalHistory::AddURI(nsIURI *aURI, PRBool aRedirect, PRBool aToplevel)
+#endif
 {
 	nsresult rv;
 
@@ -131,6 +136,20 @@ NS_IMETHODIMP MozGlobalHistory::HidePage(nsIURI *aURI)
 	return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+#ifdef MOZ_NSIGLOBALHISTORY_NSIURIP
+/* void removePage (in nsIURI aURI); */
+NS_IMETHODIMP MozGlobalHistory::RemovePage(nsIURI *aURI)
+{
+	return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+/* void removePagesFromHost (in AUTF8String aHost, in boolean aEntireDomain); */
+NS_IMETHODIMP MozGlobalHistory::RemovePagesFromHost(const nsACString &aHost, 
+						    PRBool aEntireDomain)
+{
+	return NS_ERROR_NOT_IMPLEMENTED;
+}
+#else
 /* void removePage (in string aURL); */
 NS_IMETHODIMP MozGlobalHistory::RemovePage(const char *aURL)
 {
@@ -143,6 +162,7 @@ NS_IMETHODIMP MozGlobalHistory::RemovePagesFromHost(const char *aHost,
 {
 	return NS_ERROR_NOT_IMPLEMENTED;
 }
+#endif
 
 /* void removeAllPages (); */
 NS_IMETHODIMP MozGlobalHistory::RemoveAllPages()
@@ -162,8 +182,24 @@ NS_IMETHODIMP MozGlobalHistory::GetCount(PRUint32 *aCount)
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+#ifdef MOZ_NSIGLOBALHISTORY_NSIURIP
+/* void markPageAsTyped (in AUTF8String aURI) */
+NS_IMETHODIMP MozGlobalHistory::MarkPageAsTyped(nsIURI *aURI)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+#else
 /* void markPageAsTyped (in string url); */
 NS_IMETHODIMP MozGlobalHistory::MarkPageAsTyped(const char *url)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
+#endif
+
+#ifdef MOZ_NSIBROWSERHISTORY_ADDPAGEWITHDETAILS
+/* void addPageWithDetails (in nsIURI aURI, in wstring aTitle, in long long aLastVisited); */
+NS_IMETHODIMP MozGlobalHistory::AddPageWithDetails(nsIURI *aURI, const PRUnichar *aTitle, PRInt64 aLastVisited)
+{
+	return NS_ERROR_NOT_IMPLEMENTED;
+}
+#endif
