@@ -15,13 +15,16 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-gui.h,v 1.9 2004/02/21 19:59:01 chpe Exp $
+ *  $Id: ephy-gui.h,v 1.11.4.1 2004/10/04 22:03:23 chpe Exp $
  */
 
 #ifndef EPHY_GUI_H
 #define EPHY_GUI_H
 
-#include <gtk/gtk.h>
+#include <gtk/gtkmenu.h>
+#include <gtk/gtktreeview.h>
+#include <gtk/gtktreeviewcolumn.h>
+#include <gtk/gtkwindow.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -33,6 +36,12 @@ void		ephy_gui_menu_position_under_widget	 (GtkMenu   *menu,
 							  gboolean  *push_in,
 							  gpointer  user_data);
 
+gboolean	ephy_gui_is_middle_click		 (void);
+
+gboolean	ephy_gui_select_row_by_key		 (GtkTreeView *treeview,
+							  gint column,
+							  guint32 unicode);
+
 gboolean	ephy_gui_confirm_overwrite_file	         (GtkWidget *parent,
 							  const char *filename);
 
@@ -40,6 +49,8 @@ void		ephy_gui_help				 (GtkWindow *parent,
 							  const char *file_name,
 							  const char *link_id);
 
+void		ephy_gui_set_default_window_icon	 (GtkWindow *window);
+
 G_END_DECLS
 
 #endif
