@@ -1,7 +1,7 @@
 /*
  *  Copyright (C) 2002 Jorn Baayen
- *  Copyright (C) 2003 Marco Pesenti Gritti
- *  Copyright (C) 2003 Christian Persch
+ *  Copyright (C) 2003, 2004 Marco Pesenti Gritti
+ *  Copyright (C) 2003, 2004 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-session.c,v 1.16 2004/07/31 11:58:56 chpe Exp $
+ *  $Id: ephy-session.c,v 1.20 2004/08/06 14:34:07 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -54,7 +54,9 @@
 struct EphySessionPrivate
 {
 	GList *windows;
+	GList *tool_windows;
 	GtkWidget *resume_dialog;
+	gboolean resuming;
 };
 
 #define BOOKMARKS_EDITOR_ID	"BookmarksEditor"
@@ -267,7 +269,9 @@ ephy_session_init (EphySession *session)
 	LOG ("EphySession initialising")
 
 	session->priv->windows = NULL;
+	session->priv->tool_windows = NULL;
 	session->priv->resume_dialog = NULL;
+	session->priv->resuming = FALSE;
 
 	ensure_session_directory ();
 }
@@ -289,7 +293,9 @@ ephy_session_finalize (GObject *object)
 
 	LOG ("EphySession finalising")
 
+	/* FIXME: those should be NULL already!? */
 	g_list_free (session->priv->windows);
+	g_list_free (session->priv->tool_windows);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -308,6 +314,7 @@ ephy_session_set_property (GObject *object,
 			   GParamSpec *pspec)
 {
 	/* no writeable properties */
+	g_return_if_reached ();
 }
 
 static void
@@ -346,8 +353,8 @@ ephy_session_class_init (EphySessionClass *class)
 		 g_param_spec_object ("active-window",
 		 		      "Active Window",
 		 		      "The active window",
-				       EPHY_TYPE_WINDOW,
-				       G_PARAM_READABLE));
+				      EPHY_TYPE_WINDOW,
+				      G_PARAM_READABLE));
 
 	g_type_class_add_private (object_class, sizeof (EphySessionPrivate));
 }
@@ -451,7 +458,10 @@ ephy_session_autoresume (EphySession *session)
 	if (g_file_test (saved_session, G_FILE_TEST_EXISTS)
 	    && offer_to_resume (session))
 	{
+		session->priv->resuming = TRUE;
 		retval = ephy_session_load (session, saved_session);
+		session->priv->resuming = FALSE;
+		ephy_session_save (session, SESSION_CRASHED);
 	}
 
 	g_free (saved_session);
@@ -467,11 +477,12 @@ ephy_session_close (EphySession *session)
 	LOG ("ephy_session_close")
 
 	windows	= ephy_session_get_windows (session);
-
-	/* close all windows */
 	g_list_foreach (windows, (GFunc) gtk_widget_destroy, NULL);
-
 	g_list_free (windows);
+
+	windows = g_list_copy (session->priv->tool_windows);
+	g_list_foreach (windows, (GFunc) gtk_widget_destroy, NULL);
+	g_list_free (windows);	
 }
 
 static int
@@ -507,30 +518,43 @@ write_window_geometry (xmlTextWriterPtr writer,
 	gtk_window_get_position (window, &x, &y);
 
 	/* set window properties */
-	ret = xmlTextWriterWriteFormatAttribute (writer, (xmlChar *) "x", "%d", x);
+	ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *) "x", "%d", x);
 	if (ret < 0) return ret;
 
-	ret = xmlTextWriterWriteFormatAttribute (writer, (xmlChar *) "y", "%d", y);
+	ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *) "y", "%d", y);
 	if (ret < 0) return ret;
 
-	ret = xmlTextWriterWriteFormatAttribute (writer, (xmlChar *) "width", "%d", width);
+	ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *) "width", "%d", width);
 	if (ret < 0) return ret;
 
-	ret = xmlTextWriterWriteFormatAttribute (writer, (xmlChar *) "height", "%d", height);
+	ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *) "height", "%d", height);
 	return ret;
 }
 
 static int
 write_tool_window (xmlTextWriterPtr writer,
-		   GtkWindow *window,
-		   xmlChar *id)
+		   GtkWindow *window)
 {
+	const xmlChar *id;
 	int ret;
 
-	ret = xmlTextWriterStartElement (writer, (xmlChar *) "toolwindow");
+	if (EPHY_IS_BOOKMARKS_EDITOR (window))
+	{
+		id = (const xmlChar *) BOOKMARKS_EDITOR_ID;
+	}
+	else if (EPHY_IS_HISTORY_WINDOW (window))
+	{
+		id = (const xmlChar *) HISTORY_WINDOW_ID;
+	}
+	else
+	{
+		g_return_val_if_reached (-1);
+	}
+
+	ret = xmlTextWriterStartElement (writer, (const xmlChar *) "toolwindow");
 	if (ret < 0) return ret;
 
-	ret = xmlTextWriterWriteAttribute (writer, (xmlChar *) "id", id);
+	ret = xmlTextWriterWriteAttribute (writer, (const xmlChar *) "id", id);
 	if (ret < 0) return ret;
 
 	ret = write_window_geometry (writer, window);
@@ -582,9 +606,14 @@ ephy_session_save (EphySession *session,
 	char *save_to, *tmp_file;
 	int ret;
 
+	if (session->priv->resuming)
+	{
+		return TRUE;
+	}
+
 	LOG ("ephy_sesion_save %s", filename)
 
-	if (session->priv->windows == NULL)
+	if (session->priv->windows == NULL && session->priv->tool_windows == NULL)
 	{
 		session_delete (session, filename);
 		return TRUE;
@@ -606,7 +635,7 @@ ephy_session_save (EphySession *session,
 	ret = xmlTextWriterSetIndent (writer, 1);
 	if (ret < 0) goto out;
 
-	ret = xmlTextWriterSetIndentString (writer, (xmlChar *) "  ");
+	ret = xmlTextWriterSetIndentString (writer, (const xmlChar *) "  ");
 	if (ret < 0) goto out;
 
 	START_PROFILER ("Saving session")
@@ -615,27 +644,19 @@ ephy_session_save (EphySession *session,
 	if (ret < 0) goto out;
 
 	/* create and set the root node for the session */
-	ret = xmlTextWriterStartElement (writer, (xmlChar *) "session");
+	ret = xmlTextWriterStartElement (writer, (const xmlChar *) "session");
 	if (ret < 0) goto out;
 
-	/* iterate through all the windows */
-	for (w = session->priv->windows; w != NULL; w = w->next)
+	/* iterate through all the windows */	
+	for (w = session->priv->windows; w != NULL && ret >= 0; w = w->next)
 	{
-		GtkWindow *window = w->data;
-	
-		if (EPHY_IS_WINDOW (window))
-		{
-			ret = write_ephy_window (writer, EPHY_WINDOW (window));
-		}
-		else if (EPHY_IS_BOOKMARKS_EDITOR (window))
-		{
-			ret = write_tool_window (writer, window, (xmlChar *) BOOKMARKS_EDITOR_ID);
-		}
-		else if (EPHY_IS_HISTORY_WINDOW (window))
-		{
-			ret = write_tool_window (writer, window, (xmlChar *) HISTORY_WINDOW_ID);
-		}
-		if (ret < 0) break;
+		ret = write_ephy_window (writer, EPHY_WINDOW (w->data));
+	}
+	if (ret < 0) goto out;
+
+	for (w = session->priv->tool_windows; w != NULL && ret >= 0; w = w->next)
+	{
+		ret = write_tool_window (writer, GTK_WINDOW (w->data));
 	}
 	if (ret < 0) goto out;
 
@@ -674,7 +695,7 @@ parse_embed (xmlNodePtr child, EphyWindow *window)
 
 			g_return_if_fail (window != NULL);
 
-			url = xmlGetProp (child, (xmlChar *) "url");
+			url = xmlGetProp (child, (const xmlChar *) "url");
 
 			ephy_shell_new_tab (ephy_shell, window, NULL, (char *) url,
 					    EPHY_NEW_TAB_IN_EXISTING_WINDOW |
@@ -725,27 +746,27 @@ ephy_session_load (EphySession *session,
 
 	while (child != NULL)
 	{
-		if (xmlStrEqual (child->name, (xmlChar *) "window"))
+		if (xmlStrEqual (child->name, (const xmlChar *) "window"))
 		{
 			widget = GTK_WIDGET (ephy_window_new ());
 			parse_embed (child->children, EPHY_WINDOW (widget));
 		}
-		else if (xmlStrEqual (child->name, (xmlChar *) "toolwindow"))
+		else if (xmlStrEqual (child->name, (const xmlChar *) "toolwindow"))
 		{
 			xmlChar *id;
 
-			id = xmlGetProp (child, (xmlChar *) "id");
+			id = xmlGetProp (child, (const xmlChar *) "id");
 
-			if (id && xmlStrEqual ((xmlChar *) BOOKMARKS_EDITOR_ID, id))
+			if (id && xmlStrEqual ((const xmlChar *) BOOKMARKS_EDITOR_ID, id))
 			{
 				if (!eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_BOOKMARK_EDITING))
 				{
 					widget = ephy_shell_get_bookmarks_editor (ephy_shell);
 				}
 			}
-			else if (id && xmlStrEqual ((xmlChar *) HISTORY_WINDOW_ID, id))
+			else if (id && xmlStrEqual ((const xmlChar *) HISTORY_WINDOW_ID, id))
 			{
-				if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_HISTORY))
+				if (!eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_HISTORY))
 				{
 					widget = ephy_shell_get_history_window (ephy_shell);
 				}
@@ -780,9 +801,15 @@ ephy_session_load (EphySession *session,
 
 	xmlFreeDoc (doc);
 
-	return (session->priv->windows != NULL);
+	return (session->priv->windows != NULL || session->priv->tool_windows != NULL);
 }
 
+/**
+ * ephy_session_get_windows:
+ * @ephy_session: the #EphySession
+ *
+ * Returns: the list of open #EphyWindow:s.
+ **/
 GList *
 ephy_session_get_windows (EphySession *session)
 {
@@ -803,9 +830,10 @@ void
 ephy_session_add_window (EphySession *session,
 			 GtkWindow *window)
 {
-	LOG ("ephy_session_add_window")
+	LOG ("ephy_session_add_window %p", window)
 
-	session->priv->windows = g_list_append (session->priv->windows, window);
+	session->priv->tool_windows =
+		g_list_append (session->priv->tool_windows, window);
 
 	ephy_session_save (session, SESSION_CRASHED);
 }
@@ -813,7 +841,7 @@ ephy_session_add_window (EphySession *session,
 /**
  * ephy_session_remove_window:
  * @session: a #EphySession.
- * @window: a #GtkWindow. This is either the bookmarks editor or the
+ * @window: a #GtkWindow, which must be either the bookmarks editor or the
  * history window.
  *
  * Remove a tool window from the session.
@@ -822,9 +850,10 @@ void
 ephy_session_remove_window (EphySession *session,
 			    GtkWindow *window)
 {
-	LOG ("ephy_session_remove_window")
+	LOG ("ephy_session_remove_window %p", window)
 
-	session->priv->windows = g_list_remove (session->priv->windows, window);
+	session->priv->tool_windows =
+		g_list_remove (session->priv->tool_windows, window);
 
 	ephy_session_save (session, SESSION_CRASHED);
 }
@@ -842,19 +871,15 @@ ephy_session_remove_window (EphySession *session,
 EphyWindow *
 ephy_session_get_active_window (EphySession *session)
 {
-	GList *l;
-	EphyWindow *window = NULL;
+	GList *first;
 
 	g_return_val_if_fail (EPHY_IS_SESSION (session), NULL);
 
-	for (l = session->priv->windows; l != NULL; l = l->next)
+	first = session->priv->windows;
+	if (first != NULL)
 	{
-		if (EPHY_IS_WINDOW (l->data))
-		{
-			window = EPHY_WINDOW (l->data);
-			break;
-		}
+		return EPHY_WINDOW (first->data);
 	}
 
-	return window;
+	return NULL;
 }
